package yrpg.bamp.basedata.webapi;

import kd.bos.bill.IBillWebApiPlugin;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiMapping;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import yrpg.bamp.basedata.webapi.pojo.OrgRelationDTO;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;



/**
 * 查询或者新建办事处管理与辖区绑定
 */
@ApiController(value = "orgRelation", desc = "管理组织关联信息")
@ApiMapping("/org")
public class NewOrgRelationSaveApiPlugin implements IBillWebApiPlugin {

    @ApiPostMapping(value = "/batchSaveOrgRelationInformation", desc = "管理组织关联信息")
    public CustomApiResult<Object> batchSaveOrgRelationInformation(
            @ApiParam(value = "管理组织关联信息", required = true, message = "管理组织关联信息不能为空") List<OrgRelationDTO> orgRelationList
    ) {
        DynamicObject user = BusinessDataServiceHelper.loadSingle(RequestContext.get().getCurrUserId(), "bos_user");
        Map<String, Object> map = new HashMap<>();
        ArrayList<Map<String, Object>> resultList = new ArrayList<>();
        ArrayList<OrgRelationDTO> failedList = new ArrayList<>();
        ArrayList<DynamicObject> list = new ArrayList<>();
        for (OrgRelationDTO orgRelationDTO : orgRelationList) {
            if (checkRequiredFields(orgRelationDTO)) {
                Map<String, Object> orgMap = new HashMap<>();
                orgMap.put("id", "");
                orgMap.put("number", orgRelationDTO.getMdm_code());
                orgMap.put("billStatus", false);
                orgMap.put("errors", "存在必录字段未填写");
                resultList.add(orgMap);
                failedList.add(orgRelationDTO);
                continue;
            }
            DynamicObject org = BusinessDataServiceHelper.loadSingle("bos_adminorg",
                    QFilter.of("number = ?", orgRelationDTO.getNumber()).toArray());
            if (org == null) {
                Map<String, Object> orgMap = new HashMap<>();
                orgMap.put("id", "");
                orgMap.put("number", orgRelationDTO.getMdm_code());
                orgMap.put("billStatus", false);
                orgMap.put("errors", "未查询到该组织");
                resultList.add(orgMap);
                failedList.add(orgRelationDTO);
                continue;
            }
            DynamicObject relation = BusinessDataServiceHelper.loadSingle("yrpg_mkt_sales_area_manag",
                    QFilter.of("yrpg_master_data_number = ?", orgRelationDTO.getMdm_code()).toArray());
            if (relation != null) {
                if (!relation.get("yrpg_province").equals(orgRelationDTO.getProvince()) ||
                        !relation.get("yrpg_city").equals(orgRelationDTO.getCity() == null ? "" : orgRelationDTO.getCity())) {
                    Map<String, Object> orgMap = new HashMap<>();
                    orgMap.put("id", "");
                    orgMap.put("number", orgRelationDTO.getMdm_code());
                    orgMap.put("billStatus", false);
                    orgMap.put("errors", "省市信息不能在原数据中进行更改,同步失败");
                    resultList.add(orgMap);
                    failedList.add(orgRelationDTO);
                } else {
                    if (orgRelationDTO.getEnable().equals("0")) {
                        relation.set("yrpg_end_date", new Date());
                    }
                    relation.set("yrpg_customer_class", "零售".equals(orgRelationDTO.getManagement_dimension()) ? "2" : "3");
                    list.add(relation);
                }
            } else {
                //办事处管理
                DynamicObject mktOffice = BusinessDataServiceHelper.loadSingle("yrpg_mkt_office",
                        QFilter.of("number = ?", orgRelationDTO.getNumber()).toArray());
                DynamicObject obj = QueryServiceHelper.queryOne("yrpg_mkt_sales_area_manag", "id",
                        new QFilter[]{QFilter.of("yrpg_org=?", org.get("id"))
                                .and("yrpg_end_date", QCP.not_equals, getDate(2099, 12, 31))});
                if (mktOffice == null) {
                    mktOffice = createOfficeInfo(org, user);
                }
                relation = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_sales_area_manag");
                relation.set("yrpg_master_data_number", orgRelationDTO.getMdm_code());
                relation.set("yrpg_customer_dimension", "2");
                relation.set("yrpg_goods_dimension", "1");
                relation.set("yrpg_org", org);
                relation.set("yrpg_office_id", mktOffice);
                relation.set("yrpg_start_date", obj != null ? new Date() : getDate(2020, 1, 1));
                relation.set("yrpg_end_date", getDate(2099, 12, 31));
                relation.set("creator", user);
                relation.set("auditor", user);
                relation.set("auditdate", new Date());
                relation.set("billstatus", "C");
                relation.set("yrpg_province", orgRelationDTO.getProvince());
                relation.set("yrpg_city", orgRelationDTO.getCity());
                relation.set("yrpg_customer_class", "零售".equals(orgRelationDTO.getManagement_dimension()) ? "2" : "3");
                list.add(relation);
            }
        }
        OperationResult result = SaveServiceHelper.saveOperate("yrpg_mkt_sales_area_manag", list.toArray(new DynamicObject[0]), OperateOption.create());
        if (result.isSuccess()) {
            DynamicObjectCollection successList = QueryServiceHelper.query("yrpg_mkt_sales_area_manag",
                    "id,billno,yrpg_master_data_number,yrpg_office_id.number",
                    new QFilter("id", QCP.in, result.getSuccessPkIds()).toArray());
            for (DynamicObject org : successList) {
                Map<String, Object> orgMap = new HashMap<>();
                orgMap.put("id", user.get("id"));
                orgMap.put("number", org.get("yrpg_master_data_number"));
                orgMap.put("billStatus", true);
                orgMap.put("billno", org.get("billno"));
                orgMap.put("errors", "");
                resultList.add(orgMap);
            }
            map.put("successCount", String.valueOf(result.getSuccessPkIds().size()));
            map.put("failCount", String.valueOf(failedList.size()));
            map.put("totalCount", String.valueOf(result.getSuccessPkIds().size() + failedList.size()));
            map.put("result", resultList);
            return CustomApiResult.success(map);
        } else {
            DynamicObjectCollection successList = QueryServiceHelper.query("yrpg_mkt_sales_area_manag",
                    "id,yrpg_master_data_number,yrpg_office_id.number",
                    new QFilter("id", QCP.in, result.getSuccessPkIds()).toArray());
            for (DynamicObject org : successList) {
                Map<String, Object> orgMap = new HashMap<>();
                orgMap.put("id", user.get("id"));
                orgMap.put("number", org.get("yrpg_master_data_number"));
                orgMap.put("billStatus", true);
                orgMap.put("errors", "");
                resultList.add(orgMap);
            }
            map.put("successCount", String.valueOf(result.getSuccessPkIds().size()));
            map.put("failCount", String.valueOf(result.getValidateResult().getErrorDataIndexs().size() + failedList.size()));
            map.put("totalCount", String.valueOf(result.getSuccessPkIds().size() + result.getValidateResult().getErrorDataIndexs().size() + failedList.size()));
            map.put("result", resultList);
        }
        return CustomApiResult.success(map);
    }

    private static Boolean checkRequiredFields(OrgRelationDTO orgRelationDTO) {
        return (orgRelationDTO.toString().contains("null") ||
                orgRelationDTO.toString().contains("''"));
    }

    public static DynamicObject createOfficeInfo(DynamicObject org, DynamicObject user) {
        DynamicObject yzj = BusinessDataServiceHelper.loadSingle(100000L, "bos_org");
        DynamicObject mktOffice = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_office");
        mktOffice.set("number", org.getString("yrpg_mainid"));
        mktOffice.set("name", org.getString("name"));
        mktOffice.set("yrpg_office", org);
        mktOffice.set("yrpg_is_self", "0");
        mktOffice.set("yrpg_office_type", "1");
        mktOffice.set("createorg", yzj);
        mktOffice.set("useorg", yzj);
        mktOffice.set("org", yzj);
        mktOffice.set("creator", user);
        mktOffice.set("enable", "1");
        mktOffice.set("status", "C");
        OperationResult result = SaveServiceHelper.saveOperate("yrpg_mkt_office", new DynamicObject[]{mktOffice}, OperateOption.create());
        return BusinessDataServiceHelper.loadSingle(result.getSuccessPkIds().get(0), "yrpg_mkt_office");
    }

    public static Date getDate(int year, int month, int day) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(1, year);
        calendar.set(2, month - 1);
        calendar.set(5, day);
        return calendar.getTime();
    }
}
