package yrpg.bamp.basedata.webapi;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.bill.IBillWebApiPlugin;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.operate.result.OperateErrorInfo;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.validate.ValidateResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiMapping;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.sec.user.utils.UserOperationUtils;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import kd.bos.util.StringUtils;
import org.jetbrains.annotations.NotNull;
import yrpg.bamp.basedata.webapi.pojo.UserDTO;


import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;


import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 先保存人员再保存人员档案
 */
@ApiController(value = "user", desc = "人员")
@ApiMapping("/user")
public class NewUserSaveApiPlugin implements IBillWebApiPlugin {


    private static final DynamicObject yzj = BusinessDataServiceHelper.loadSingle(100000L, "bos_org");

    private static final Map<String, String> userStatusMap = new HashMap<>();

    static {
        userStatusMap.put("1", "正式员工");
        userStatusMap.put("2", "实习员工");
        userStatusMap.put("3", "聘用员工");
        userStatusMap.put("4", "劳务派遣员工");
        userStatusMap.put("5", "离退休员工");
        userStatusMap.put("6", "离职员工");
        userStatusMap.put("7", "不在岗位员工");
    }

    @ApiPostMapping(value = "/saveUserInformation", desc = "主数据人员信息")
    public CustomApiResult<Object> saveUserInformation(
            @ApiParam(value = "工号", required = true, message = "人员工号不能为空") String number,
            @ApiParam(value = "姓名", required = true, message = "人员姓名不能为空") String name,
            @ApiParam(value = "手机", required = true, message = "人员手机不能为空") String phone,
            @ApiParam(value = "性别") String gender,
            @ApiParam(value = "邮箱") String email,
            @ApiParam(value = "入职日期") Date entryDate,
            @ApiParam(value = "员工组") String employeeGroup,
            @ApiParam(value = "管理组织") String managementOrg,
            @ApiParam(value = "一级销售队伍") String firstSalesTeam,
            @ApiParam(value = "二级销售队伍") String secondSalesTeam,
            @ApiParam(value = "部门", required = true, message = "人员部门不能为空") String department,
            @ApiParam(value = "职位", required = true, message = "人员职位不能为空") String position,
            @ApiParam(value = "使用状态", required = true, message = "人员使用状态不能为空") String enable,
            @ApiParam(value = "主数据编码", required = true, message = "人员主数据编码不能为空") String masterUserid,
            @ApiParam(value = "操作类型", required = true, message = "操作类型不能为空") String operationType
    ) {
        DynamicObject user = BusinessDataServiceHelper.loadSingle("bos_user",
                QFilter.of("number = ?", number).toArray());
        if (operationType.equals("3") || operationType.equals("4")) {
            if (user != null) {
                String ableStatus = operationType.equals("3") ? "enable" : "disable";
                OperationServiceHelper.executeOperate(ableStatus, "bos_user", new DynamicObject[]{user}, OperateOption.create());
                return CustomApiResult.success("success");
            } else {
                return CustomApiResult.fail("", "未查询到该人员,人员编码为:{" + number + "}");
            }
        }
        DynamicObject dpt = BusinessDataServiceHelper.loadSingle("bos_adminorg", "id",
                QFilter.of("yrpg_mainid = ?", department).toArray());
        DynamicObject morg = BusinessDataServiceHelper.loadSingle("bos_adminorg", "id",
                QFilter.of("yrpg_mainid = ?", managementOrg).toArray());
        if (dpt == null) {
            return CustomApiResult.fail("", "未查询到该部门,部门编码为:{" + department + "}");
        }
        if (user != null) {
            user.set("phone", phone);
            user.set("gender", gender);
            user.set("email", email);
            user.set("entrydate", entryDate);
            user.set("employeegroup", employeeGroup);
            user.set("masteruserid", masterUserid);
            user.set("firstsalesteam", firstSalesTeam);
            user.set("secondsalesteam", secondSalesTeam);
            user.set("managementorg", morg);
            setName(user, name);
            user.set("name", name);
            DynamicObjectCollection entryentity = user.getDynamicObjectCollection("entryentity");
            DynamicObject org = entryentity.get(0);
            org.set("dpt", dpt);
            org.set("position", position);
        } else {
            user = BusinessDataServiceHelper.newDynamicObject("bos_user");
            user.set("number", number);
            user.set("name", name);
            user.set("phone", phone);
            user.set("gender", gender);
            user.set("email", email);
            user.set("entrydate", entryDate);
            user.set("employeegroup", employeeGroup);
            user.set("masteruserid", masterUserid);
            user.set("firstsalesteam", firstSalesTeam);
            user.set("secondsalesteam", secondSalesTeam);
            user.set("managementorg", morg);
            user.set("enable", enable);
            user.set("status", "C");
            addUserTypes(user);
            DynamicObjectCollection entryentity = user.getDynamicObjectCollection("entryentity");
            DynamicObject org = entryentity.addNew();
            org.set("dpt", dpt);
            org.set("position", position);
        }

        OperationResult result = SaveServiceHelper.saveOperate("bos_user", new DynamicObject[]{user}, OperateOption.create());
        if (result.isSuccess()) {
            DynamicObject mktUser = BusinessDataServiceHelper.loadSingle("yrpg_mkt_user",
                    QFilter.of("number = ?", number).toArray());
            if (mktUser != null) {
                mktUser.set("name", name);
                mktUser.set("yrpg_short_name", UserOperationUtils.getFirstSpellByName(name));
                mktUser.set("yrpg_long_name", UserOperationUtils.getFullSpellByName(name));
                mktUser.set("yrpg_telephone_number", phone);
                mktUser.set("yrpg_gender", gender);
                mktUser.set("yrpg_email", email);
                mktUser.set("yrpg_entrydate", entryDate);
                mktUser.set("yrpg_employeegroup", employeeGroup);
                mktUser.set("yrpg_masteruserid", masterUserid);
                mktUser.set("yrpg_firstsalesteam", firstSalesTeam);
                mktUser.set("yrpg_secondsalesteam", secondSalesTeam);
                mktUser.set("yrpg_managementorg", morg);
                mktUser.set("yrpg_dep", dpt.getString("name"));
                mktUser.set("enable", enable);
            } else {
                mktUser = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_user");
                mktUser.set("number", number);
                mktUser.set("name", name);
                mktUser.set("yrpg_short_name", UserOperationUtils.getFirstSpellByName(name));
                mktUser.set("yrpg_long_name", UserOperationUtils.getFullSpellByName(name));
                mktUser.set("yrpg_telephone_number", phone);
                mktUser.set("yrpg_gender", gender);
                mktUser.set("yrpg_email", email);
                mktUser.set("yrpg_entrydate", entryDate);
                mktUser.set("yrpg_employeegroup", employeeGroup);
                mktUser.set("yrpg_masteruserid", masterUserid);
                mktUser.set("yrpg_firstsalesteam", firstSalesTeam);
                mktUser.set("yrpg_secondsalesteam", secondSalesTeam);
                mktUser.set("yrpg_managementorg", morg);
                mktUser.set("createorg", yzj);
                mktUser.set("useorg", yzj);
                mktUser.set("org", yzj);
                mktUser.set("yrpg_dep", dpt.getString("name"));
                mktUser.set("yrpg_source_code", number);
                mktUser.set("enable", enable);
                mktUser.set("status", "C");
                mktUser.set("yrpg_src", "2");
                mktUser.set("yrpg_usertype", "1");
            }
            SaveServiceHelper.saveOperate("yrpg_mkt_user", new DynamicObject[]{mktUser}, OperateOption.create());
            return CustomApiResult.success("success");
        } else {
            List<ValidateResult> validateErrors = result.getValidateResult().getValidateErrors();
            if (!validateErrors.isEmpty()) {
                List<OperateErrorInfo> allErrorInfo = validateErrors.get(0).getAllErrorInfo();
                if (!allErrorInfo.isEmpty()) {
                    StringBuilder message = new StringBuilder();
                    for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                        message.append(operateErrorInfo.getMessage());
                    }
                    return CustomApiResult.fail("", message.toString());
                } else {
                    return CustomApiResult.fail("", result.getMessage());
                }
            } else {
                return CustomApiResult.fail("", result.getMessage());
            }
        }
    }

    @ApiPostMapping(value = "/batchSaveUserInformation", desc = "主数据人员信息")
    public CustomApiResult<Object> batchSaveUserInformation(
            @ApiParam(value = "人员数据", required = true, message = "人员数据不能为空") List<UserDTO> userList,
            @ApiParam(value = "操作类型", required = true, message = "操作类型不能为空") String operationType
    ) throws UnsupportedEncodingException {
        if (userList.size() > 500) {
            return CustomApiResult.fail("", "请一次性处理500条以下的数据");
        }
        Map<String, Object> map = new HashMap<>();
        ArrayList<Map<String, Object>> resultList = new ArrayList<>();
        ArrayList<UserDTO> failedList = new ArrayList<>();
        ArrayList<DynamicObject> list = new ArrayList<>();
        String prefix = RequestContext.get().getAccountId().equals("1862465440106677248") ? "HR" : "";
        //费控扩展
        //员工id-用于记录保存成功后的收款信息保存；
        Map<String, Object> useridMap = new HashMap<>();
        //校验付款信息集合
        Map<String, String> payErrMap = new HashMap<String, String>();
        for (UserDTO userDTO : userList) {
            if (checkRequiredFields(userDTO)) {
                Map<String, Object> userMap = new HashMap<>();
                userMap.put("id", "");
                userMap.put("number", userDTO.getMasterUserid());
                userMap.put("billStatus", false);
                userMap.put("errors", "存在人员必录字段未填写");
                resultList.add(userMap);
                failedList.add(userDTO);
                continue;
            }
            String name = userDTO.getName().replace(" ", "");
            DynamicObject user = BusinessDataServiceHelper.loadSingleFromCache("bos_user",
                    QFilter.of("number = ?", userDTO.getNumber()).toArray());
            DynamicObject dpt = BusinessDataServiceHelper.loadSingleFromCache("bos_adminorg", "id",
                    QFilter.of("yrpg_mainid = ?", userDTO.getDepartment()).toArray());
            DynamicObject morg = BusinessDataServiceHelper.loadSingleFromCache("bos_adminorg", "id",
                    QFilter.of("yrpg_mainid = ?", userDTO.getManagementOrg()).toArray());
            String position = userDTO.getPosition() != null && !userDTO.getPosition().trim().isEmpty() ? userDTO.getPosition() :
                    userStatusMap.get(userDTO.getEmployeeGroup()) == null ? "无" : userStatusMap.get(userDTO.getEmployeeGroup());

            //付款信息校验

            checkPayInfo(payErrMap, userDTO);

            if (user != null) {
                useridMap.put(userDTO.getNumber(), user.getPkValue());
                user.set("phone", userDTO.getPhone());
                user.set("gender", userDTO.getGender());
                user.set("email", userDTO.getEmail());
                user.set("entrydate", userDTO.getEntryDate());
                user.set("employeegroup", userDTO.getEmployeeGroup());
                user.set("masteruserid", userDTO.getMasterUserid());
                user.set("firstsalesteam", userDTO.getFirstSalesTeam());
                user.set("secondsalesteam", userDTO.getSecondSalesTeam());
                user.set("managementorg", morg);
                user.set("enable", userDTO.getEnable());
//                user.set("idcard", aesDecrypt(userDTO.getIdcard()));
                user.set("idcard", Base64.getDecoder().decode(userDTO.getIdcard().getBytes(defaultCharset)));
                user.set("rank", userDTO.getRank());
                setName(user, name);
                user.set("name", name);
                DynamicObjectCollection entryentity = user.getDynamicObjectCollection("entryentity");
                Iterator<DynamicObject> iterator = entryentity.iterator();
                while (iterator.hasNext()) {
                    DynamicObject org = iterator.next();
                    if (org.get("ispartjob").equals(false)) {
                        org.set("dpt", dpt);
                        org.set("position", position);
                    } else {
                        if (morg != null) {
                            org.set("dpt", morg);
                            org.set("position", position);
                        } else {
                            iterator.remove();
                        }
                    }
                }
                if (morg != null && entryentity.size() == 1) {
                    DynamicObject manageOrg = entryentity.addNew();
                    manageOrg.set("dpt", morg);
                    manageOrg.set("ispartjob", true);
                    manageOrg.set("position", position);
                }
            } else {
                user = BusinessDataServiceHelper.newDynamicObject("bos_user");
                user.set("number", userDTO.getNumber());
                user.set("name", name);
                user.set("phone", userDTO.getPhone());
                user.set("gender", userDTO.getGender());
                user.set("email", userDTO.getEmail());
                user.set("entrydate", userDTO.getEntryDate());
                user.set("employeegroup", userDTO.getEmployeeGroup());
                user.set("masteruserid", userDTO.getMasterUserid());
                user.set("firstsalesteam", userDTO.getFirstSalesTeam());
                user.set("secondsalesteam", userDTO.getSecondSalesTeam());
                user.set("managementorg", morg);
                user.set("enable", userDTO.getEnable());
                user.set("idcard", Base64.getDecoder().decode(userDTO.getIdcard().getBytes(defaultCharset)));
                user.set("rank", userDTO.getRank());
                user.set("status", "C");
                addUserTypes(user);
                DynamicObjectCollection entryentity = user.getDynamicObjectCollection("entryentity");
                DynamicObject org = entryentity.addNew();
                org.set("dpt", dpt);
                org.set("ispartjob", false);
                org.set("position", position);
                if (morg != null) {
                    DynamicObject manageOrg = entryentity.addNew();
                    manageOrg.set("dpt", morg);
                    manageOrg.set("ispartjob", true);
                    manageOrg.set("position", position);
                }
            }
            list.add(user);
        }
        int payErrCount = 0;

        OperationResult result = SaveServiceHelper.saveOperate("bos_user", list.toArray(new DynamicObject[0]), OperateOption.create());
        if (result.isSuccess()) {
            DynamicObjectCollection successList = QueryServiceHelper.query("bos_user", "id,number",
                    new QFilter("id", QCP.in, result.getSuccessPkIds()).toArray());
            List<String> numberList = successList.stream().map(o -> o.getString("number"))
                    .collect(Collectors.toList());
            for (DynamicObject user : successList) {
                Map<String, Object> userMap = new HashMap<>();
                userMap.put("id", user.get("id"));
                userMap.put("number", prefix + user.get("number"));
                userMap.put("billStatus", true);
                userMap.put("errors", "");
                //判断付款信息是否校验成功
                if (payErrMap.get(user.get("number")) != null) {
                    userMap.put("billStatus", false);
                    userMap.put("errors", payErrMap.get(user.get("number")));
                    payErrCount++;
                }
                //判断
                resultList.add(userMap);
                useridMap.put(user.getString("number"), user.get("id"));
            }
            //计算成功失败条数
            map.put("successCount", String.valueOf(result.getSuccessPkIds().size() - payErrCount));
            map.put("failCount", String.valueOf(failedList.size() + payErrCount));
            map.put("totalCount", String.valueOf(result.getSuccessPkIds().size() + failedList.size()));
            map.put("result", resultList);

            userList.removeAll(failedList);
            ArrayList<DynamicObject> mktUserList = new ArrayList<>();
            for (UserDTO userDTO : userList) {
                String name = userDTO.getName().replace(" ", "");
                if (numberList.contains(userDTO.getNumber())) {
                    DynamicObject mktUser = BusinessDataServiceHelper.loadSingle("yrpg_mkt_user",
                            QFilter.of("number = ?", userDTO.getNumber()).toArray());
                    DynamicObject dpt = BusinessDataServiceHelper.loadSingle("bos_adminorg", "id",
                            QFilter.of("yrpg_mainid = ?", userDTO.getDepartment()).toArray());
                    DynamicObject morg = BusinessDataServiceHelper.loadSingle("bos_adminorg", "id",
                            QFilter.of("yrpg_mainid = ?", userDTO.getManagementOrg()).toArray());
                    if (mktUser != null) {
                        mktUser.set("name", name);
                        mktUser.set("yrpg_short_name", UserOperationUtils.getFirstSpellByName(name));
                        mktUser.set("yrpg_long_name", UserOperationUtils.getFullSpellByName(name));
                        mktUser.set("yrpg_telephone_number", userDTO.getPhone());
                        mktUser.set("yrpg_gender", userDTO.getGender());
                        mktUser.set("yrpg_email", userDTO.getEmail());
                        mktUser.set("yrpg_entrydate", userDTO.getEntryDate());
                        mktUser.set("yrpg_employeegroup", userDTO.getEmployeeGroup());
                        mktUser.set("yrpg_masteruserid", userDTO.getMasterUserid());
                        mktUser.set("yrpg_firstsalesteam", userDTO.getFirstSalesTeam());
                        mktUser.set("yrpg_secondsalesteam", userDTO.getSecondSalesTeam());
                        mktUser.set("yrpg_managementorg", morg);
                        mktUser.set("yrpg_dep", dpt.getString("name"));
                        mktUser.set("enable", userDTO.getEnable());
                    } else {
                        mktUser = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_user");
                        mktUser.set("number", userDTO.getNumber());
                        mktUser.set("name", name);
                        mktUser.set("yrpg_short_name", UserOperationUtils.getFirstSpellByName(name));
                        mktUser.set("yrpg_long_name", UserOperationUtils.getFullSpellByName(name));
                        mktUser.set("yrpg_telephone_number", userDTO.getPhone());
                        mktUser.set("yrpg_gender", userDTO.getGender());
                        mktUser.set("yrpg_email", userDTO.getEmail());
                        mktUser.set("yrpg_entrydate", userDTO.getEntryDate());
                        mktUser.set("yrpg_employeegroup", userDTO.getEmployeeGroup());
                        mktUser.set("yrpg_masteruserid", userDTO.getMasterUserid());
                        mktUser.set("yrpg_firstsalesteam", userDTO.getFirstSalesTeam());
                        mktUser.set("yrpg_secondsalesteam", userDTO.getSecondSalesTeam());
                        mktUser.set("yrpg_managementorg", morg);
                        mktUser.set("createorg", yzj);
                        mktUser.set("useorg", yzj);
                        mktUser.set("org", yzj);
                        mktUser.set("yrpg_dep", dpt.getString("name"));
                        mktUser.set("yrpg_source_code", userDTO.getNumber());
                        mktUser.set("enable", userDTO.getEnable());
                        mktUser.set("status", "C");
                        mktUser.set("yrpg_src", "2");
                        mktUser.set("yrpg_usertype", "1");
                    }
                    mktUserList.add(mktUser);
                }
            }
            SaveServiceHelper.saveOperate("yrpg_mkt_user", mktUserList.toArray(new DynamicObject[0]), OperateOption.create());
            //费控扩展  --更新收款信息
            syncPaybank(userList, useridMap, map);
            return CustomApiResult.success(map);
        } else {
            DynamicObjectCollection successList = QueryServiceHelper.query("bos_user", "id,number",
                    new QFilter("id", QCP.in, result.getSuccessPkIds()).toArray());
            List<String> numberList = successList.stream().map(o -> o.getString("number"))
                    .collect(Collectors.toList());
            for (DynamicObject user : successList) {
                Map<String, Object> userMap = new HashMap<>();
                userMap.put("id", user.get("id"));
                userMap.put("number", prefix + user.get("number"));
                userMap.put("billStatus", true);
                userMap.put("errors", "");
                //判断付款信息是否校验成功
                if (payErrMap.get(user.get("number")) != null) {
                    userMap.put("billStatus", false);
                    userMap.put("errors", payErrMap.get(user.get("number")));
                    payErrCount++;
                }
                resultList.add(userMap);
            }
            map.put("successCount", String.valueOf(result.getSuccessPkIds().size() - payErrCount));
            map.put("failCount", String.valueOf(result.getValidateResult().getErrorDataIndexs().size() + failedList.size() + payErrCount));
            map.put("totalCount", String.valueOf(result.getSuccessPkIds().size() + result.getValidateResult().getErrorDataIndexs().size() + failedList.size()));
            List<OperateErrorInfo> allErrorInfo = result.getValidateResult().getValidateErrors().get(0).getAllErrorInfo();
            if (allErrorInfo.size() == 1 && !allErrorInfo.get(0).getMessage().contains("：")) {
                Map<String, Object> userMap = new HashMap<>();
                userMap.put("id", "");
                userMap.put("number", prefix + result.getBillNos().toString().replace("}", "").split("=")[1]);
                userMap.put("billStatus", false);
                userMap.put("errors", allErrorInfo.get(0).getMessage());
                resultList.add(userMap);
            } else {
                for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                    Map<String, Object> userMap = new HashMap<>();
                    userMap.put("id", "");
                    String[] split = operateErrorInfo.getMessage().split("：");
                    userMap.put("number", prefix + split[0]);
                    userMap.put("billStatus", false);
                    userMap.put("errors", split[1]);
                    resultList.add(userMap);
                }
                Map<String, Long> collect = resultList.stream()
                        .filter(o -> o.get("billStatus").equals(false))
                        .map(o -> o.get("number").toString())
                        .collect(Collectors.toList()).stream()
                        .collect(Collectors.groupingBy(Function.identity(), Collectors.counting()));
                List<String> duplicates = collect.entrySet().stream()
                        .filter(entry -> entry.getValue() > 1)
                        .map(Map.Entry::getKey)
                        .collect(Collectors.toList());
                List<Map<String, Object>> last = resultList.stream()
                        .filter(o -> o.get("billStatus").equals(false))
                        .filter(o -> duplicates.contains(o.get("number").toString()))
                        .collect(Collectors.toList());
                resultList.removeAll(last);
                for (String duplicate : duplicates) {
                    StringBuilder msg = new StringBuilder();
                    for (Map<String, Object> err : last) {
                        if (err.get("number").equals(duplicate)) {
                            msg.append(err.get("errors"));
                        }
                    }
                    Map<String, Object> userMap = new HashMap<>();
                    userMap.put("id", "");
                    userMap.put("number", duplicate);
                    userMap.put("billStatus", false);
                    userMap.put("errors", msg);
                    resultList.add(userMap);
                }
            }
            map.put("result", resultList);

            userList.removeAll(failedList);
            ArrayList<DynamicObject> mktUserList = new ArrayList<>();
            for (UserDTO userDTO : userList) {
                String name = userDTO.getName().replace(" ", "");
                if (numberList.contains(userDTO.getNumber())) {
                    DynamicObject mktUser = BusinessDataServiceHelper.loadSingle("yrpg_mkt_user",
                            QFilter.of("number = ?", userDTO.getNumber()).toArray());
                    DynamicObject dpt = BusinessDataServiceHelper.loadSingle("bos_adminorg", "id",
                            QFilter.of("yrpg_mainid = ?", userDTO.getDepartment()).toArray());
                    DynamicObject morg = BusinessDataServiceHelper.loadSingle("bos_adminorg", "id",
                            QFilter.of("yrpg_mainid = ?", userDTO.getManagementOrg()).toArray());
                    if (mktUser != null) {
                        mktUser.set("name", name);
                        mktUser.set("yrpg_short_name", UserOperationUtils.getFirstSpellByName(name));
                        mktUser.set("yrpg_long_name", UserOperationUtils.getFullSpellByName(name));
                        mktUser.set("yrpg_telephone_number", userDTO.getPhone());
                        mktUser.set("yrpg_gender", userDTO.getGender());
                        mktUser.set("yrpg_email", userDTO.getEmail());
                        mktUser.set("yrpg_entrydate", userDTO.getEntryDate());
                        mktUser.set("yrpg_employeegroup", userDTO.getEmployeeGroup());
                        mktUser.set("yrpg_masteruserid", userDTO.getMasterUserid());
                        mktUser.set("yrpg_firstsalesteam", userDTO.getFirstSalesTeam());
                        mktUser.set("yrpg_secondsalesteam", userDTO.getSecondSalesTeam());
                        mktUser.set("yrpg_managementorg", morg);
                        mktUser.set("yrpg_dep", dpt.getString("name"));
                        mktUser.set("enable", userDTO.getEnable());
                    } else {
                        mktUser = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_user");
                        mktUser.set("number", userDTO.getNumber());
                        mktUser.set("name", name);
                        mktUser.set("yrpg_short_name", UserOperationUtils.getFirstSpellByName(name));
                        mktUser.set("yrpg_long_name", UserOperationUtils.getFullSpellByName(name));
                        mktUser.set("yrpg_telephone_number", userDTO.getPhone());
                        mktUser.set("yrpg_gender", userDTO.getGender());
                        mktUser.set("yrpg_email", userDTO.getEmail());
                        mktUser.set("yrpg_entrydate", userDTO.getEntryDate());
                        mktUser.set("yrpg_employeegroup", userDTO.getEmployeeGroup());
                        mktUser.set("yrpg_masteruserid", userDTO.getMasterUserid());
                        mktUser.set("yrpg_firstsalesteam", userDTO.getFirstSalesTeam());
                        mktUser.set("yrpg_secondsalesteam", userDTO.getSecondSalesTeam());
                        mktUser.set("yrpg_managementorg", morg);
                        mktUser.set("createorg", yzj);
                        mktUser.set("useorg", yzj);
                        mktUser.set("org", yzj);
                        mktUser.set("yrpg_dep", dpt.getString("name"));
                        mktUser.set("yrpg_source_code", userDTO.getNumber());
                        mktUser.set("enable", userDTO.getEnable());
                        mktUser.set("status", "C");
                        mktUser.set("yrpg_src", "2");
                        mktUser.set("yrpg_usertype", "1");
                    }
                    mktUserList.add(mktUser);
                }
            }
            SaveServiceHelper.saveOperate("yrpg_mkt_user", mktUserList.toArray(new DynamicObject[0]), OperateOption.create());
        }
        syncPaybank(userList, useridMap, map);


        return CustomApiResult.success(map);

    }

    //校验收款信息
    private void checkPayInfo(Map<String, String> payErrMap, UserDTO userDTO) {
        String payerbank = userDTO.getPayerbank();
        String payerbankNumber = "";
        String payeraccount = "";

        if (!StringUtils.isEmpty(payerbank)) {
            JSONArray payerbanks = JSONObject.parseArray(payerbank);
            if (payerbanks.size() > 0) {
                payerbankNumber = payerbanks.getJSONObject(0).getString("payerbankNumber");
                payeraccount = payerbanks.getJSONObject(0).getString("payerbank");
                if (StringUtils.isEmpty(payerbankNumber)) {
                    payErrMap.put(userDTO.getNumber(), "银行账户为空！");
                }
                if (StringUtils.isEmpty(payeraccount)) {
                    payErrMap.put(userDTO.getNumber(), "联行号为空！");
                }
                DynamicObject bebank = BusinessDataServiceHelper.loadSingleFromCache("bd_bebank", "id",
                        QFilter.of("union_number = ?", payerbankNumber).toArray());
                if (bebank == null) {
                    payErrMap.put(userDTO.getNumber(), "联行号" + payerbankNumber + "不存在！");
                }
            }
        }
    }

    //新增更新收款信息
    private void syncPaybank(List<UserDTO> userList, Map<String, Object> useridMap, Map<String, Object> map) {

        //收款信息  --费控项目改造
        DynamicObject currency = BusinessDataServiceHelper.loadSingleFromCache("bd_currency", "id",
                QFilter.of("number = ?", "CNY").toArray());
        Map<Long, String> userNumMap = new HashMap<Long, String>();
        List<DynamicObject> payinfos = new ArrayList<DynamicObject>();
        for (UserDTO userDTO : userList) {
            String number = userDTO.getNumber();
            if (useridMap.get(number) != null) {
                userNumMap.put((Long) useridMap.get(number), number);
                DynamicObject payeer = BusinessDataServiceHelper.loadSingle("er_payeer",
                        QFilter.of("number = ?", userDTO.getNumber()).toArray());
                String payerbank = userDTO.getPayerbank();
                String payerbankNumber = "";
                String payeraccount = "";

                if (!StringUtils.isEmpty(payerbank)) {
                    JSONArray payerbanks = JSONObject.parseArray(payerbank);
                    if (payerbanks.size() > 0) {
                        payerbankNumber = payerbanks.getJSONObject(0).getString("payerbankNumber");
                        payeraccount = payerbanks.getJSONObject(0).getString("payerbank");
                    }
                }
                if (StringUtils.isEmpty(payerbankNumber)) {
                    continue;
                }
                if (payeer == null) {
                    DynamicObject user = BusinessDataServiceHelper.loadSingleFromCache(useridMap.get(number), "bos_user");
                    payeer = BusinessDataServiceHelper.newDynamicObject("er_payeer");
                    payeer.set("status", "C");
                    payeer.set("isemployee", true);
                    payeer.set("isdefault", true);
                    payeer.set("enable", "1");
                    DynamicObject org = BusinessDataServiceHelper.loadSingleFromCache("bos_adminorg", "id",
                            QFilter.of("yrpg_mainid = ?", userDTO.getDepartment()).toArray());
                    payeer.set("createorg", yzj);
                    payeer.set("org", yzj);
                    payeer.set("useorg", yzj);
                    payeer.set("ctrlstrategy", "5");
                    payeer.set("currency", currency);
                    DynamicObjectCollection userid = payeer.getDynamicObjectCollection("userid");
                    DynamicObject userObj = new DynamicObject(userid.getDynamicObjectType());
                    userObj.set("fbasedataId", user);
                    userid.add(userObj);
                    payeer.set("userid", userid);
                    payeer.set("payer", user);
                    payeer.set("payeraccountname", user.get("name"));
                    payeer.set("outpayer", user.get("name"));
                    payeer.set("name", user.get("name"));
                    payeer.set("number", user.get("number"));

                }
                //收款账户
                payeer.set("payeraccount", payeraccount);

                DynamicObject bebank = BusinessDataServiceHelper.loadSingleFromCache("bd_bebank", "id",
                        QFilter.of("union_number = ?", payerbankNumber).toArray());
                payeer.set("payerbank", bebank);
                payinfos.add(payeer);
            }
        }
        if (payinfos.size() > 0) {
            Map<String, String> payErrMap = new HashMap<String, String>();
            OperationResult payInfoResult = SaveServiceHelper.saveOperate("er_payeer", payinfos.toArray(new DynamicObject[0]), OperateOption.create());
        }
        //--费控项目改造
    }

    private static void addUserTypes(DynamicObject user) {
        DynamicObject userType = BusinessDataServiceHelper.loadSingle("bos_usertype", "id",
                QFilter.of("number = 'PT01'").toArray());
        DynamicObjectCollection userTypes = user.getDynamicObjectCollection("usertypes");
        userTypes.clear();
        userTypes.addNew().set("fbasedataid", userType);
    }

    private static void setName(DynamicObject user, String name) {
        if (user.getString("name").equals(name)) {
            return;
        }
        String fullPinyin = UserOperationUtils.getFullSpellByName(name);
        String simplePinyin = UserOperationUtils.getFirstSpellByName(name);
        DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle("bos_user", "id,username",
                QFilter.of("username = ?", fullPinyin).toArray());
        if (dynamicObject == null) {
            user.set("username", fullPinyin);
        } else {
            user.set("username", UserOperationUtils.getUserNameByFormatedFullPinyin(user.getLong("id"),
                    fullPinyin, null));
        }
        user.set("fullpinyin", fullPinyin);
        user.set("simplepinyin", simplePinyin);
    }

    private static Boolean checkRequiredFields(UserDTO userDTO) {
        return (userDTO.toString().contains("null") ||
                userDTO.toString().contains("''"));
    }

    private static final String secretKey = BusinessDataServiceHelper.loadSingle("yrpg_user_config",
            new QFilter("number", QCP.equals, "UserConfig").toArray()).getString("key");
    private static final String iv = BusinessDataServiceHelper.loadSingle("yrpg_user_config",
            new QFilter("number", QCP.equals, "UserConfig").toArray()).getString("iv");
    private static final String algorithmStr = "AES/CBC/ISO10126Padding";
    private static final String defaultCharset = "UTF-8";

//    public static String aesDecrypt(String content) {
//        try {
//            Key key = new SecretKeySpec(secretKey.getBytes(defaultCharset), "AES");
//
//            byte[] ivBytes = iv.getBytes();
//            IvParameterSpec ivSpec = new IvParameterSpec(ivBytes);
//
//            Security.addProvider(new BouncyCastleProvider());
//            Cipher cipher = Cipher.getInstance(algorithmStr);
//            cipher.init(Cipher.DECRYPT_MODE, key, ivSpec);
//
//            byte[] decodeBase64 = Base64.getDecoder().decode(content.getBytes(defaultCharset));
//
//            byte[] original = cipher.doFinal(decodeBase64);
//            return new String(original, defaultCharset).trim();
//        } catch (Exception e) {
//            return null;
//        }
//    }

//    @NotNull
//    private static Cipher getCipher(byte[] ivBytes, Key key) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, InvalidAlgorithmParameterException {
//
//    }
}
